@@ -373,6 +373,27 @@ class MerginClient:
         resp = self.get('/v1/user/' + self.username())
         return json.load(resp)
 
+    def set_project_access(self, project_path, access):
+        """
+        Updates access for given project.
+        :param project_path: project full name (<namespace>/<name>)
+        :param access: dict <readersnames, writersnames, ownersnames> -> list of str username we want to give access to
+        """
+        if not self._user_info:
+            raise Exception("Authentication required")
+
+        params = {"access": access}
+        path = "/v1/project/%s" % project_path
+        url = urllib.parse.urljoin(self.url, urllib.parse.quote(path))
+        json_headers = {'Content-Type': 'application/json'}
+        try:
+            request = urllib.request.Request(url, data=json.dumps(params).encode(), headers=json_headers, method="PUT")
+            self._do_request(request)
+        except Exception as e:
+            detail = f"Project path: {project_path}"
+            raise ClientError(str(e), detail)
+
+
     def push_project(self, directory):
         """
         Upload local changes to the repository.
@@ -471,3 +492,17 @@ class MerginClient:
         params = {'path': file_path}
         resp = self.get("/v1/resource/changesets/{}/{}".format(project_path, version), params)
         return json.load(resp)
+
+    def get_projects_by_names(self, projects):
+        """ Returns JSON with projects' info for list of required projects.
+        The schema of the returned information is the same as the response from projects_list().
+
+        This is useful when we have a couple of Mergin projects available locally and we want to
+        find out their status at once (e.g. whether there is a new version on the server).
+
+        :param projects: list of projects in the form 'namespace/project_name'
+        :type projects: List[String]
+        """
+
+        resp = self.post("/v1/project/by_names", {"projects": projects}, {"Content-Type": "application/json"})
+        return json.load(resp)
