@@ -11,6 +11,8 @@ from ..utils import generate_checksum
 SERVER_URL = os.environ.get('TEST_MERGIN_URL')
 API_USER = os.environ.get('TEST_API_USERNAME')
 USER_PWD = os.environ.get('TEST_API_PASSWORD')
+API_USER2 = os.environ.get('TEST_API_USERNAME2')
+USER_PWD2 = os.environ.get('TEST_API_PASSWORD2')
 TMP_DIR = tempfile.gettempdir()
 TEST_DATA_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'test_data')
 CHANGED_SCHEMA_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'modified_schema')
@@ -22,8 +24,16 @@ def toggle_geodiff(enabled):
 
 @pytest.fixture(scope='function')
 def mc():
-    assert SERVER_URL and SERVER_URL.rstrip('/') != 'https://public.cloudmergin.com' and API_USER and USER_PWD
-    return MerginClient(SERVER_URL, login=API_USER, password=USER_PWD)
+    return create_client(API_USER, USER_PWD)
+
+@pytest.fixture(scope='function')
+def mc2():
+    return create_client(API_USER2, USER_PWD2)
+
+
+def create_client(user, pwd):
+    assert SERVER_URL and SERVER_URL.rstrip('/') != 'https://public.cloudmergin.com' and user and pwd
+    return MerginClient(SERVER_URL, login=user, password=pwd)
 
 
 def cleanup(mc, project, dirs):
@@ -32,6 +42,11 @@ def cleanup(mc, project, dirs):
         mc.delete_project(project)
     except ClientError:
         pass
+    remove_folders(dirs)
+
+
+def remove_folders(dirs):
+    # clean given directories
     for d in dirs:
         if os.path.exists(d):
             shutil.rmtree(d)
@@ -314,7 +329,6 @@ def test_sync_diff(mc, push_geodiff_enabled, pull_geodiff_enabled):
     toggle_geodiff(True)
 
 
-@pytest.mark.skip(reason="currently fails on test.dev instance due to server bug")
 def test_list_of_push_changes(mc):
     PUSH_CHANGES_SUMMARY = "{'base.gpkg': {'geodiff_summary': [{'table': 'gpkg_contents', 'insert': 0, 'update': 1, 'delete': 0}, {'table': 'simple', 'insert': 1, 'update': 0, 'delete': 0}]}}"
 
@@ -337,7 +351,7 @@ def test_list_of_push_changes(mc):
 
     mc._auth_session["expire"] = datetime.now().replace(tzinfo=pytz.utc) - timedelta(days=1)
     mc._auth_params = None
-    with pytest.raises(ClientError, match="You don't have the permission to access the requested resource. It is either read-protected or not readable by the server."):
+    with pytest.raises(ClientError, match="You do not have permissions for this project"):
         mc.project_status(project_dir)
 
 
@@ -461,6 +475,7 @@ def test_empty_file_in_subdir(mc):
     mc.pull_project(project_dir_2)
     assert os.path.exists(os.path.join(project_dir_2, 'subdir2', 'empty2.txt'))
 
+
 def test_clone_project(mc):
     test_project = 'test_clone_project'
     test_project_fullname = API_USER + '/' + test_project
@@ -483,3 +498,167 @@ def test_clone_project(mc):
     mc.clone_project(test_project_fullname, cloned_project_name, API_USER)
     projects = mc.projects_list(flag='created')
     assert any(p for p in projects if p['name'] == cloned_project_name and p['namespace'] == API_USER)
+
+
+def test_set_read_write_access(mc):
+    test_project = 'test_set_read_write_access'
+    test_project_fullname = API_USER + '/' + test_project
+
+    # cleanups
+    project_dir = os.path.join(TMP_DIR, test_project, API_USER)
+    cleanup(mc, test_project_fullname, [project_dir])
+
+    # create new (empty) project on server
+    mc.create_project(test_project)
+
+    # Add writer access to another client
+    project_info = get_project_info(mc, API_USER, test_project)
+    access = project_info['access']
+    access['writersnames'].append(API_USER2)
+    access['readersnames'].append(API_USER2)
+    mc.set_project_access(test_project_fullname, access)
+
+    # check access
+    project_info = get_project_info(mc, API_USER, test_project)
+    access = project_info['access']
+    assert API_USER2 in access['writersnames']
+    assert API_USER2 in access['readersnames']
+
+
+def test_available_storage_validation(mc):
+    """
+    Testing of storage limit - applies to user pushing changes into own project (namespace matching username).
+    This test also tests giving read and write access to another user. Additionally tests also uploading of big file.
+    """
+    test_project = 'test_available_storage_validation'
+    test_project_fullname = API_USER + '/' + test_project
+
+    # cleanups
+    project_dir = os.path.join(TMP_DIR, test_project, API_USER)
+    cleanup(mc, test_project_fullname, [project_dir])
+
+    # create new (empty) project on server
+    mc.create_project(test_project)
+
+    # download project
+    mc.download_project(test_project_fullname, project_dir)
+
+    # get user_info about storage capacity
+    user_info = mc.user_info()
+    storage_remaining = user_info['storage'] - user_info['disk_usage']
+
+    # generate dummy data (remaining storage + extra 1024b)
+    dummy_data_path = project_dir + "/data"
+    file_size = storage_remaining + 1024
+    _generate_big_file(dummy_data_path, file_size)
+
+    # try to upload
+    got_right_err = False
+    try:
+        mc.push_project(project_dir)
+    except ClientError as e:
+        # Expecting "Storage limit has been reached" error msg.
+        assert str(e).startswith("Storage limit has been reached")
+        got_right_err = True    
+    assert got_right_err
+
+    # Expecting empty project
+    project_info = get_project_info(mc, API_USER, test_project)
+    assert project_info['version'] == 'v0'
+    assert project_info['disk_usage'] == 0
+
+
+def test_available_storage_validation2(mc, mc2):
+    """
+    Testing of storage limit - should not be applied for user pushing changes into project with different namespace.
+    This should cover the exception of mergin-py-client that a user can push changes to someone else's project regardless
+    the user's own storage limitation. Of course, other limitations are still applied (write access, owner of
+    a modified project has to have enough free storage).
+
+    Therefore NOTE that there are following assumptions:
+        - API_USER2's free storage >= API_USER's free storage + 1024b (size of changes to be pushed)
+        - both accounts should ideally have a free plan
+    """
+    test_project = 'test_available_storage_validation2'
+    test_project_fullname = API_USER2 + '/' + test_project
+
+    # cleanups
+    project_dir = os.path.join(TMP_DIR, test_project, API_USER)
+    cleanup(mc, test_project_fullname, [project_dir])
+    cleanup(mc2, test_project_fullname, [project_dir])
+
+    # create new (empty) project on server
+    mc2.create_project(test_project)
+
+    # Add writer access to another client
+    project_info = get_project_info(mc2, API_USER2, test_project)
+    access = project_info['access']
+    access['writersnames'].append(API_USER)
+    access['readersnames'].append(API_USER)
+    mc2.set_project_access(test_project_fullname, access)
+
+    # download project
+    mc.download_project(test_project_fullname, project_dir)
+
+    # get user_info about storage capacity
+    user_info = mc.user_info()
+    storage_remaining = user_info['storage'] - user_info['disk_usage']
+
+    # generate dummy data (remaining storage + extra 1024b)
+    dummy_data_path = project_dir + "/data"
+    file_size = storage_remaining + 1024
+    _generate_big_file(dummy_data_path, file_size)
+
+    # try to upload
+    mc.push_project(project_dir)
+
+    # Check project content
+    project_info = mc.project_info(test_project_fullname)
+    assert len(project_info["files"]) == 1
+    assert project_info["disk_usage"] == file_size
+
+    # remove dummy big file from a disk
+    remove_folders([project_dir])
+
+
+def get_project_info(mc, namespace, project_name):
+    """
+    Returns first (and suppose to be just one) project info dict of project matching given namespace and name.
+    :param mc: MerginClient instance
+    :param namespace: project's namespace
+    :param project_name: project's name
+    :return: dict with project info
+    """
+    projects = mc.projects_list(flag='created')
+    test_project_list = [p for p in projects if p['name'] == project_name and p['namespace'] == namespace]
+    assert len(test_project_list) == 1
+    return test_project_list[0]
+
+
+def _generate_big_file(filepath, size):
+    """
+    generate big binary file with the specified size in bytes
+    :param filepath: full filepath
+    :param size: the size in bytes
+    """
+    with open(filepath, 'wb') as fout:
+        fout.write(b"\0" * size)
+
+
+def test_get_projects_by_name(mc):
+    """ Test server 'bulk' endpoint for projects' info"""
+    test_projects = {
+        "projectA": f"{API_USER}/projectA",
+        "projectB": f"{API_USER}/projectB"
+    }
+
+    for name, full_name in test_projects.items():
+        cleanup(mc, full_name, [])
+        mc.create_project(name)
+
+    resp = mc.get_projects_by_names(list(test_projects.values()))
+    assert len(resp) == len(test_projects)
+    for name, full_name in test_projects.items():
+        assert full_name in resp
+        assert resp[full_name]["name"] == name
+        assert resp[full_name]["version"] == 'v0'
